@@ -4,25 +4,26 @@
  *  
  *  Copyright (C) 2011 Peng Wu <alexepico@gmail.com>
  *  
- *  This program is free software; you can redistribute it and/or modify
+ *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
- * 
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "chewing_large_table.h"
 #include <assert.h>
 #include "pinyin_phrase2.h"
+#include "pinyin_phrase3.h"
 #include "pinyin_parser2.h"
+#include "zhuyin_parser2.h"
 
 
 /* internal class definition */
@@ -661,7 +662,7 @@ int ChewingArrayIndexLevel<phrase_length>::remove_index
 
 
 /* load text method */
-bool ChewingLargeTable::load_text(FILE * infile) {
+bool ChewingLargeTable::load_text(FILE * infile, TABLE_PHONETIC_TYPE type) {
     char pinyin[256];
     char phrase[256];
     phrase_token_t token;
@@ -679,15 +680,27 @@ bool ChewingLargeTable::load_text(FILE * infile) {
 
         glong len = g_utf8_strlen(phrase, -1);
 
-        PinyinDirectParser2 parser;
         ChewingKeyVector keys;
         ChewingKeyRestVector key_rests;
 
         keys = g_array_new(FALSE, FALSE, sizeof(ChewingKey));
         key_rests = g_array_new(FALSE, FALSE, sizeof(ChewingKeyRest));
 
-        pinyin_option_t options = USE_TONE;
-        parser.parse(options, keys, key_rests, pinyin, strlen(pinyin));
+        switch (type) {
+        case PINYIN_TABLE: {
+            PinyinDirectParser2 parser;
+            pinyin_option_t options = USE_TONE;
+            parser.parse(options, keys, key_rests, pinyin, strlen(pinyin));
+            break;
+        }
+
+        case ZHUYIN_TABLE: {
+            ZhuyinDirectParser2 parser;
+            pinyin_option_t options = USE_TONE | FORCE_TONE;
+            parser.parse(options, keys, key_rests, pinyin, strlen(pinyin));
+            break;
+        }
+        };
 
         if (len != keys->len) {
             fprintf(stderr, "ChewingLargeTable::load_text:%s\t%s\t%u\t%ld\n",
