@@ -4,25 +4,24 @@
  *  
  *  Copyright (C) 2006-2007 Peng Wu
  *  
- *  This program is free software; you can redistribute it and/or modify
+ *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
- * 
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "phrase_index.h"
 #include "pinyin_custom2.h"
 
-using namespace pinyin;
+namespace pinyin{
 
 bool PhraseItem::set_n_pronunciation(guint8 n_prouns){
     m_chunk.set_content(sizeof(guint8), &n_prouns, sizeof(guint8));
@@ -37,7 +36,7 @@ bool PhraseItem::get_nth_pronunciation(size_t index, ChewingKey * keys,
     bool retval = m_chunk.get_content
         (offset, keys, phrase_length * sizeof(ChewingKey));
     if ( !retval )
-	return retval;
+        return retval;
     return m_chunk.get_content
         (offset + phrase_length * sizeof(ChewingKey), &freq , sizeof(guint32));
 }
@@ -71,7 +70,7 @@ bool PhraseItem::add_pronunciation(ChewingKey * keys, guint32 delta){
             (keys, (ChewingKey *)chewing_begin, phrase_length)) {
             /* found the exact match pinyin keys. */
 
-	    /* protect against total_freq overflow. */
+            /* protect against total_freq overflow. */
             if (delta > 0 && total_freq > total_freq + delta)
                 return false;
 
@@ -107,8 +106,7 @@ bool PhraseItem::set_phrase_string(guint8 phrase_length, ucs4_t * phrase){
     return true;
 }
 
-void PhraseItem::increase_pronunciation_possibility(pinyin_option_t options,
-                                                    ChewingKey * keys,
+void PhraseItem::increase_pronunciation_possibility(ChewingKey * keys,
                                                     gint32 delta){
     guint8 phrase_length = get_phrase_length();
     guint8 npron = get_n_pronunciation();
@@ -117,23 +115,22 @@ void PhraseItem::increase_pronunciation_possibility(pinyin_option_t options,
     guint32 total_freq = 0;
 
     for (int i = 0; i < npron; ++i) {
-	char * chewing_begin = buf_begin + offset +
-	    i * (phrase_length * sizeof(ChewingKey) + sizeof(guint32));
-	guint32 * freq = (guint32 *)(chewing_begin +
+        char * chewing_begin = buf_begin + offset +
+            i * (phrase_length * sizeof(ChewingKey) + sizeof(guint32));
+        guint32 * freq = (guint32 *)(chewing_begin +
                                      phrase_length * sizeof(ChewingKey));
-	total_freq += *freq;
+        total_freq += *freq;
 
-	if (0 == pinyin_compare_with_ambiguities2
-            (options, keys,
-             (ChewingKey *)chewing_begin, phrase_length)) {
+        if (0 == pinyin_compare_with_tones(keys, (ChewingKey *)chewing_begin,
+                                           phrase_length)) {
 
-	    /* protect against total_freq overflow. */
-	    if (delta > 0 && total_freq > total_freq + delta)
-		return;
+            /* protect against total_freq overflow. */
+            if (delta > 0 && total_freq > total_freq + delta)
+                return;
 
-	    *freq += delta;
-	    total_freq += delta;
-	}
+            *freq += delta;
+            total_freq += delta;
+        }
     }
 }
 
@@ -146,24 +143,24 @@ int SubPhraseIndex::add_unigram_frequency(phrase_token_t token, guint32 delta){
     table_offset_t offset;
     guint32 freq;
     bool result = m_phrase_index.get_content
-	((token & PHRASE_MASK) 
-	 * sizeof(table_offset_t), &offset, sizeof(table_offset_t));
+        ((token & PHRASE_MASK) 
+         * sizeof(table_offset_t), &offset, sizeof(table_offset_t));
 
     if ( !result )
-	return ERROR_OUT_OF_RANGE;
+        return ERROR_OUT_OF_RANGE;
 
     if ( 0 == offset )
         return ERROR_NO_ITEM;
 
     result = m_phrase_content.get_content
-	(offset + sizeof(guint8) + sizeof(guint8), &freq, sizeof(guint32));
+        (offset + sizeof(guint8) + sizeof(guint8), &freq, sizeof(guint32));
 
     if ( !result )
         return ERROR_FILE_CORRUPTION;
 
     //protect total_freq overflow
     if ( delta > 0 && m_total_freq > m_total_freq + delta )
-	return ERROR_INTEGER_OVERFLOW;
+        return ERROR_INTEGER_OVERFLOW;
 
     freq += delta;
     m_total_freq += delta;
@@ -178,11 +175,11 @@ int SubPhraseIndex::get_phrase_item(phrase_token_t token, PhraseItem & item){
     guint8 n_prons;
     
     bool result = m_phrase_index.get_content
-	((token & PHRASE_MASK) 
-	 * sizeof(table_offset_t), &offset, sizeof(table_offset_t));
+        ((token & PHRASE_MASK) 
+         * sizeof(table_offset_t), &offset, sizeof(table_offset_t));
 
     if ( !result )
-	return ERROR_OUT_OF_RANGE;
+        return ERROR_OUT_OF_RANGE;
 
     if ( 0 == offset )
         return ERROR_NO_ITEM;
@@ -193,7 +190,7 @@ int SubPhraseIndex::get_phrase_item(phrase_token_t token, PhraseItem & item){
     
     result = m_phrase_content.get_content(offset+sizeof(guint8), &n_prons, sizeof(guint8));
     if ( !result ) 
-	return ERROR_FILE_CORRUPTION;
+        return ERROR_FILE_CORRUPTION;
 
     size_t length = phrase_item_header + phrase_length * sizeof ( ucs4_t ) + n_prons * ( phrase_length * sizeof (ChewingKey) + sizeof(guint32) );
     item.m_chunk.set_chunk((char *)m_phrase_content.begin() + offset, length, NULL);
@@ -203,10 +200,10 @@ int SubPhraseIndex::get_phrase_item(phrase_token_t token, PhraseItem & item){
 int SubPhraseIndex::add_phrase_item(phrase_token_t token, PhraseItem * item){
     table_offset_t offset = m_phrase_content.size();
     if ( 0 == offset )
-	offset = 8;
+        offset = 8;
     m_phrase_content.set_content(offset, item->m_chunk.begin(), item->m_chunk.size());
     m_phrase_index.set_content((token & PHRASE_MASK) 
-			       * sizeof(table_offset_t), &offset, sizeof(table_offset_t));
+                               * sizeof(table_offset_t), &offset, sizeof(table_offset_t));
     m_total_freq += item->get_unigram_frequency();
     return ERROR_OK;
 }
@@ -224,7 +221,7 @@ int SubPhraseIndex::remove_phrase_item(phrase_token_t token, PhraseItem * & item
 
     const table_offset_t zero_const = 0;
     m_phrase_index.set_content((token & PHRASE_MASK)
-			       * sizeof(table_offset_t), &zero_const, sizeof(table_offset_t));
+                               * sizeof(table_offset_t), &zero_const, sizeof(table_offset_t));
     m_total_freq -= item->get_unigram_frequency();
     return ERROR_OK;
 }
@@ -232,13 +229,13 @@ int SubPhraseIndex::remove_phrase_item(phrase_token_t token, PhraseItem * & item
 bool FacadePhraseIndex::load(guint8 phrase_index, MemoryChunk * chunk){
     SubPhraseIndex * & sub_phrases = m_sub_phrase_indices[phrase_index];
     if ( !sub_phrases ){
-	sub_phrases = new SubPhraseIndex;
+        sub_phrases = new SubPhraseIndex;
     }
 
     m_total_freq -= sub_phrases->get_phrase_index_total_freq();
     bool retval = sub_phrases->load(chunk, 0, chunk->size());
     if ( !retval )
-	return retval;
+        return retval;
     m_total_freq += sub_phrases->get_phrase_index_total_freq();
     return retval;
 }
@@ -247,7 +244,7 @@ bool FacadePhraseIndex::store(guint8 phrase_index, MemoryChunk * new_chunk){
     table_offset_t end;
     SubPhraseIndex * & sub_phrases = m_sub_phrase_indices[phrase_index];
     if ( !sub_phrases )
-	return false;
+        return false;
     
     sub_phrases->store(new_chunk, 0, end);
     return true;
@@ -256,7 +253,7 @@ bool FacadePhraseIndex::store(guint8 phrase_index, MemoryChunk * new_chunk){
 bool FacadePhraseIndex::unload(guint8 phrase_index){
     SubPhraseIndex * & sub_phrases = m_sub_phrase_indices[phrase_index];
     if ( !sub_phrases )
-	return false;
+        return false;
     m_total_freq -= sub_phrases->get_phrase_index_total_freq();
     delete sub_phrases;
     sub_phrases = NULL;
@@ -328,18 +325,18 @@ bool FacadePhraseIndex::merge_with_mask(guint8 phrase_index,
 
 
 bool SubPhraseIndex::load(MemoryChunk * chunk, 
-			  table_offset_t offset, table_offset_t end){
+                          table_offset_t offset, table_offset_t end){
     //save the memory chunk
     if ( m_chunk ){
-	delete m_chunk;
-	m_chunk = NULL;
+        delete m_chunk;
+        m_chunk = NULL;
     }
     m_chunk = chunk;
     
     char * buf_begin = (char *)chunk->begin();
     chunk->get_content(offset, &m_total_freq, sizeof(guint32));
     offset += sizeof(guint32);
-    table_offset_t index_one, index_two, index_three;
+    table_offset_t index_one = 0, index_two = 0, index_three = 0;
     chunk->get_content(offset, &index_one, sizeof(table_offset_t));
     offset += sizeof(table_offset_t);
     chunk->get_content(offset, &index_two, sizeof(table_offset_t));
@@ -350,7 +347,7 @@ bool SubPhraseIndex::load(MemoryChunk * chunk,
     g_return_val_if_fail(*(buf_begin + index_two - 1) == c_separate, FALSE);
     g_return_val_if_fail(*(buf_begin + index_three - 1) == c_separate, FALSE);
     m_phrase_index.set_chunk(buf_begin + index_one, 
-			     index_two - 1 - index_one, NULL);
+                             index_two - 1 - index_one, NULL);
     m_phrase_content.set_chunk(buf_begin + index_two, 
                                index_three - 1 - index_two, NULL);
     g_return_val_if_fail( index_three <= end, FALSE);
@@ -358,7 +355,7 @@ bool SubPhraseIndex::load(MemoryChunk * chunk,
 }
 
 bool SubPhraseIndex::store(MemoryChunk * new_chunk, 
-			   table_offset_t offset, table_offset_t& end){
+                           table_offset_t offset, table_offset_t& end){
     new_chunk->set_content(offset, &m_total_freq, sizeof(guint32));
     table_offset_t index = offset + sizeof(guint32);
         
@@ -432,7 +429,8 @@ bool SubPhraseIndex::diff(SubPhraseIndex * oldone, PhraseIndexLogger * logger){
 }
 
 bool SubPhraseIndex::merge(PhraseIndexLogger * logger){
-    LOG_TYPE log_type; phrase_token_t token;
+    LOG_TYPE log_type = LOG_INVALID_RECORD;
+    phrase_token_t token = null_token;
     MemoryChunk oldchunk, newchunk;
     PhraseItem olditem, newitem, item, * tmpitem;
 
@@ -513,10 +511,11 @@ bool SubPhraseIndex::merge(PhraseIndexLogger * logger){
     return true;
 }
 
-bool FacadePhraseIndex::load_text(guint8 phrase_index, FILE * infile){
+bool FacadePhraseIndex::load_text(guint8 phrase_index, FILE * infile,
+                                  TABLE_PHONETIC_TYPE type){
     SubPhraseIndex * & sub_phrases = m_sub_phrase_indices[phrase_index];
     if ( !sub_phrases ){
-	sub_phrases = new SubPhraseIndex;
+        sub_phrases = new SubPhraseIndex;
     }
 
     char pinyin[256];
@@ -534,46 +533,58 @@ bool FacadePhraseIndex::load_text(guint8 phrase_index, FILE * infile){
         if (4 != num)
             continue;
 
-	if (feof(infile))
-	    break;
+        if (feof(infile))
+            break;
 
         assert(PHRASE_INDEX_LIBRARY_INDEX(token) == phrase_index );
 
-	glong written;
-	ucs4_t * phrase_ucs4 = g_utf8_to_ucs4(phrase, -1, NULL, 
+        glong written;
+        ucs4_t * phrase_ucs4 = g_utf8_to_ucs4(phrase, -1, NULL, 
                                               &written, NULL);
 	
-	if ( 0 == cur_token ){
-	    cur_token = token;
-	    item_ptr->set_phrase_string(written, phrase_ucs4);
-	}
+        if ( 0 == cur_token ){
+            cur_token = token;
+            item_ptr->set_phrase_string(written, phrase_ucs4);
+        }
 
-	if ( cur_token != token ){
-	    add_phrase_item( cur_token, item_ptr);
-	    delete item_ptr;
-	    item_ptr = new PhraseItem;
-	    cur_token = token;
-	    item_ptr->set_phrase_string(written, phrase_ucs4);
-	}
+        if ( cur_token != token ){
+            add_phrase_item( cur_token, item_ptr);
+            delete item_ptr;
+            item_ptr = new PhraseItem;
+            cur_token = token;
+            item_ptr->set_phrase_string(written, phrase_ucs4);
+        }
 
-    pinyin_option_t options = USE_TONE;
-	PinyinDirectParser2 parser;
-	ChewingKeyVector keys = g_array_new(FALSE, FALSE, sizeof(ChewingKey));
-	ChewingKeyRestVector key_rests =
+        ChewingKeyVector keys = g_array_new(FALSE, FALSE, sizeof(ChewingKey));
+        ChewingKeyRestVector key_rests =
             g_array_new(FALSE, FALSE, sizeof(ChewingKeyRest));
 
-	parser.parse(options, keys, key_rests, pinyin, strlen(pinyin));
-	
-	if (item_ptr->get_phrase_length() == keys->len) {
+        switch (type) {
+        case PINYIN_TABLE: {
+            PinyinDirectParser2 parser;
+            pinyin_option_t options = USE_TONE;
+            parser.parse(options, keys, key_rests, pinyin, strlen(pinyin));
+            break;
+        }
+
+        case ZHUYIN_TABLE: {
+            ZhuyinDirectParser2 parser;
+            pinyin_option_t options = USE_TONE | FORCE_TONE;
+            parser.parse(options, keys, key_rests, pinyin, strlen(pinyin));
+            break;
+        }
+        };
+
+        if (item_ptr->get_phrase_length() == keys->len) {
             item_ptr->add_pronunciation((ChewingKey *)keys->data, freq);
         } else {
             fprintf(stderr, "FacadePhraseIndex::load_text:%s\t%s\n",
                     pinyin, phrase);
         }
 
-	g_array_free(keys, TRUE);
-	g_array_free(key_rests, TRUE);
-	g_free(phrase_ucs4);
+        g_array_free(keys, TRUE);
+        g_array_free(key_rests, TRUE);
+        g_free(phrase_ucs4);
     }
 
     add_phrase_item( cur_token, item_ptr);
@@ -706,8 +717,6 @@ bool FacadePhraseIndex::mask_out(guint8 phrase_index,
     return retval;
 }
 
-namespace pinyin{
-
 
 static bool _peek_header(PhraseIndexLogger * logger,
                          guint32 & old_total_freq){
@@ -743,7 +752,8 @@ bool _compute_new_header(PhraseIndexLogger * logger,
                          phrase_token_t value,
                          guint32 & new_total_freq) {
 
-    LOG_TYPE log_type; phrase_token_t token;
+    LOG_TYPE log_type = LOG_INVALID_RECORD;
+    phrase_token_t token = null_token;
     MemoryChunk oldchunk, newchunk;
     PhraseItem olditem, newitem;
 
@@ -808,7 +818,8 @@ static bool _mask_out_records(PhraseIndexLogger * oldlogger,
                               phrase_token_t mask,
                               phrase_token_t value,
                               PhraseIndexLogger * newlogger) {
-    LOG_TYPE log_type; phrase_token_t token;
+    LOG_TYPE log_type = LOG_INVALID_RECORD;
+    phrase_token_t token = null_token;
     MemoryChunk oldchunk, newchunk;
 
     while(oldlogger->has_next_record()) {
