@@ -4,19 +4,18 @@
  *  
  *  Copyright (C) 2010 Peng Wu
  *  
- *  This program is free software; you can redistribute it and/or modify
+ *  This program is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation, either version 3 of the License, or
  *  (at your option) any later version.
- * 
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
- *  
+ *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <stdio.h>
@@ -34,11 +33,14 @@ static GOptionEntry entries[] =
 
 bool generate_binary_files(const char * pinyin_table_filename,
                            const char * phrase_table_filename,
-                           const pinyin_table_info_t * phrase_files) {
+                           const pinyin_table_info_t * phrase_files,
+                           TABLE_PHONETIC_TYPE type) {
     /* generate pinyin index*/
-    pinyin_option_t options = USE_TONE;
-    ChewingLargeTable chewing_table(options);
-    PhraseLargeTable2 phrase_table;
+    ChewingLargeTable2 pinyin_table;
+    pinyin_table.attach(pinyin_table_filename, ATTACH_READWRITE|ATTACH_CREATE);
+
+    PhraseLargeTable3 phrase_table;
+    phrase_table.attach(phrase_table_filename, ATTACH_READWRITE|ATTACH_CREATE);
 
     /* generate phrase index */
     FacadePhraseIndex phrase_index;
@@ -61,25 +63,15 @@ bool generate_binary_files(const char * pinyin_table_filename,
             exit(ENOENT);
         }
 
-        chewing_table.load_text(tablefile);
+        pinyin_table.load_text(tablefile, type);
         fseek(tablefile, 0L, SEEK_SET);
         phrase_table.load_text(tablefile);
         fseek(tablefile, 0L, SEEK_SET);
-        phrase_index.load_text(i, tablefile);
+        phrase_index.load_text(i, tablefile, type);
         fclose(tablefile);
         g_free(filename);
     }
 
-    MemoryChunk * new_chunk = new MemoryChunk;
-    chewing_table.store(new_chunk);
-    new_chunk->save(pinyin_table_filename);
-    chewing_table.load(new_chunk);
-    
-    new_chunk = new MemoryChunk;
-    phrase_table.store(new_chunk);
-    new_chunk->save(phrase_table_filename);
-    phrase_table.load(new_chunk);
-
     phrase_index.compact();
 
     if (!save_phrase_index(phrase_files, &phrase_index))
@@ -117,15 +109,16 @@ int main(int argc, char * argv[]){
     const pinyin_table_info_t * phrase_files =
         system_table_info.get_default_tables();
 
+    TABLE_PHONETIC_TYPE type = system_table_info.get_table_phonetic_type();
     generate_binary_files(SYSTEM_PINYIN_INDEX,
                           SYSTEM_PHRASE_INDEX,
-                          phrase_files);
+                          phrase_files, type);
 
     phrase_files = system_table_info.get_addon_tables();
 
     generate_binary_files(ADDON_SYSTEM_PINYIN_INDEX,
                           ADDON_SYSTEM_PHRASE_INDEX,
-                          phrase_files);
+                          phrase_files, type);
 
     return 0;
 }
